function combineTrcFiles(varargin)
%combineTrcFiles
%
%   combineTrcFiles() combines two or more .trc files into one .trc file.
%
%   The input arguments to this function are a cell array containing .trc
%   filenames, e.g., files={'file1.trc','file2.trc'}, and the output .trc
%   filename (optional). If an output filename is not specified, a default
%   name (combinedFile.trc) is used.
%   
%   The function returns a single, combined, .trc file.
%   
%   Example function calls:
%   combineTrcFiles({'file1.trc','file2.trc'})
%   combineTrcFiles({'file1.trc','file2.trc'},'file3.trc')

%check input arguments and populate variables accordingly
Nin=nargin;%number of input arguments
if Nin==1
    inData=varargin{1};%input filenames
    outFile='combinedFile.trc';%default output filename
else
    inData=varargin{1};%input filenames
    outFile=varargin{2};%output filename
end

units='m';%data units
%get marker names; note that we are making the assumption that both .trc
%files share the same marker names, therefore only need to run the
%following snippet on one marker file.
fid=fopen(inData{1},'r');%open file again
%read all strings (cell array)
txt=textscan(fid,'%s');
fclose(fid);%close file
%find appropriate indices to get marker names
for itxt=1:length(txt{1})
    if strcmp(txt{1}(itxt),'Time')
        indxS=itxt;
        break;
    end
end
for jtxt=1:length(txt{1})
    if strcmp(txt{1}(jtxt),'X1')
        indxE=jtxt;
        break;
    end
end
%get marker names
markerNames=txt{1}(indxS+1:indxE-1);
nMarkers=length(markerNames);%number of markers
%loop through and get data from each marker file
for ii=1:length(inData)
    %open .trc file
    fid=fopen(inData{ii},'r');
    nl=fgetl(fid);nl=fgetl(fid);nl=fgetl(fid);%skip first couple of lines
    values=sscanf(nl, '%f %f %f %f');%scan values
    samplingFrequency(ii)=values(1);%sampling frequency (Hz)
    numframes=values(3);%number of frames
    numcolumns=values(4)*3+2;%number of data columns
    numOfMarkers=values(4);%number of markers in file
    nl=fgetl(fid);nl=fgetl(fid);nl=fgetl(fid);%again, skip un-wanted lines
    d=fscanf(fid,'%f',[numcolumns,numframes])';%read data
    fclose(fid);%close file
    markerLocations=[d(:,3:end)];%skip Frame and Time columns, hence 3:end
    %since there is a chance that the files have different lengths, store
    %the data in a structure array
    eval(['data.expData',num2str(ii),'=markerLocations;'])
end
%sanity check: ensure both files have the same sampling frequency
value=numel(unique(samplingFrequency));
if (value > 1)
    error('Sampling rates must be identical!')
end
%combine data
combinedData=[];%initialize
for ii=1:length(inData)
    eval(['combinedData=[combinedData;data.expData',num2str(ii),'];'])
end
%prepare frame and time column
[r,c]=size(combinedData);
frame=linspace(1,r,r)';%frame numbers
%update number of frames
numframes=length(frame);
%time vector (sec); assumes that time starts at 0 sec
time=[(0:r-1)./samplingFrequency(1)]';
%append to frame and time to combinedData matrix
combinedData=[frame time combinedData];
%--------------------------------------------------------------------------
%Note: This snippet of code has been adopted from code written by Glen
%Lichtwark from the Universty of Queensland and Rod Barrett from Griffin
%University. Their complete code is available on simtk.org
%--------------------------------------------------------------------------
%prepare to write data to file
header1='Frame#\tTime\t';
header2='\t\t';
textFormat='%i\t%2.4f\t';
outData=[combinedData(:,1) combinedData(:,2)];%frame and time columns
combinedData(:,1:2) = [];
counter=1;%counter
for jj=1:nMarkers
    header1=[header1 markerNames{jj} '\t\t\t'];    
    header2=[header2 'X' num2str(jj) '\t' 'Y' num2str(jj)...
        '\t' 'Z' num2str(jj) '\t'];
    textFormat=[textFormat '%f\t%f\t%f\t'];
    outData=[outData combinedData(:,counter:counter+2)];
    counter=counter+3;
end
header1=[header1 '\n'];
header2=[header2 '\n\n'];
textFormat=[textFormat '\n'];
%open output file
fid=fopen(outFile,'w');
%write the header data
fprintf(fid,'PathFileType\t4\t(X/Y/Z)\t %s\n',outFile);
txt1='DataRate\tCameraRate\tNumFrames\tNumMarkers\tUnits\tOrigDataRate';
txt2='\tOrigDataStartFrame\tOrigNumFrames\n';
fprintf(fid,[txt1 txt2]);
txt3='%d\t%d\t%d\t%d\t%s\t%d\t%d\t%d\n';
fprintf(fid,txt3,samplingFrequency(1),samplingFrequency(1),numframes,...
    nMarkers,units,samplingFrequency(1),1,numframes); 
fprintf(fid,header1);
fprintf(fid,header2);
%write the output marker data
fprintf(fid,textFormat,outData');
%close the file
fclose(fid);