function downSampleTrcFile(inputFile,downSampleFactor)

% Read in TRC file
[markerNames,markerData,time] = readTrcFile(inputFile);

% Down Sample
newIndices = 1:downSampleFactor:length(time);
newMarkerData = markerData(newIndices,:);
newTime = time(newIndices);

% Output down sampled file
outFileName = strrep(inputFile,'.trc','_downSampled.trc');
outputTRCFile(outFileName,newMarkerData,newTime,markerNames);

end


%% TRC read/write functions

function varargout = readTrcFile(inFileName)
%readTrcFile
%
%   readTrcFile() opens a .trc file and extracts marker names and
%   locations for use in the main kinematic calibration routine.
%   
%   The input argument to the function is the name of the .trc
%   file.
%   
%   The function returns the marker names (in cell array), a matrix
%   containing marker locations, and a time vector.
%   
%   Example function call:
%   [markerNames,markerLocations,time]=readTrcFile(markerFileName);

%open .trc file
fid = fopen(inFileName,'r');

%skip first couple of lines
nl = fgetl(fid); nl = fgetl(fid); nl = fgetl(fid);

%extract desired variables
values = sscanf(nl, '%f %f %f %f');%scan values
numframes = values(3);%number of frames
numcolumns = values(4)*3+2;%number of data columns
numOfMarkers = values(4);%number of markers

%again, skip un-wanted lines
nl = fgetl(fid); nl = fgetl(fid); nl = fgetl(fid);

%read data
data=fscanf(fid,'%f',[numcolumns,numframes])';
%close file
fclose(fid);


expMarkerLocations = data(:,3:end);

%time vector
time = data(:,2);

%open file again
fid = fopen(inFileName,'r');

%read all strings (cell array)
txt = textscan(fid,'%s');

%close file
fclose(fid);

%find appropriate indices to get marker names
for itxt=1:length(txt{1})
    if strcmp(txt{1}(itxt),'Time')
        indxS=itxt;
        break;
    end
end
for jtxt=1:length(txt{1})
    if strcmp(txt{1}(jtxt),'X1')
        indxE=jtxt;
        break;
    end
end

%get marker names
markerNames = txt{1}(indxS+1:indxE-1);

%prepare function output arguments
if (nargout==2)
    varargout={markerNames,expMarkerLocations};
else
    varargout={markerNames,expMarkerLocations,time};
end

end %end readTrcFile

function outputTRCFile(outFile, MarkerData,TimeVector,markerNames)

nMarkers = length(markerNames);

Header = HeaderTRC;

rate = 1/mean(diff(TimeVector));
AllMarkersTRC = MarkerData;

MarkerLabels = markerNames;
Header = strrep(Header,'DATARATE_INS',sprintf('%.12f',rate));
Header = strrep(Header,'CAMRATE_INS',sprintf('%.12f',rate));
Header = strrep(Header,'ORIGRATE_INS',sprintf('%.12f',rate));
Header = strrep(Header,'NFRAMES_INS',sprintf('%d',length(TimeVector)));
Header = strrep(Header,'NMARKERS_INS',sprintf('%d',nMarkers));
Header = strrep(Header,'UNITS_INS','m');
Header = strrep(Header,'FILENAME',outFile);

% Create Marker Labels String
RemoveXYZLabel = 0;
if length(MarkerLabels)==(nMarkers*3)
    MarkerLabels=MarkerLabels([1:3:end]);
    RemoveXYZLabel = 1;
end
MarkerLabelString ='';
XYZString ='';
for m=1:nMarkers
    if length(MarkerLabels)<m
        label = ['Marker' num2str(m)];
    else
        label = MarkerLabels{m};
        if RemoveXYZLabel
            label = strrep(label,'_x','');
        end
    end
    XYZString = [XYZString sprintf('X%d\tY%d\tZ%d\t',m,m,m)];
    MarkerLabelString = [MarkerLabelString sprintf('%s\t\t\t',label)];
end
Header = strrep(Header,'FIRSTMARKERNAME_INS',MarkerLabelString);
Header = strrep(Header,'XYZ_INS',XYZString);

% Create Data string
DataString = '';
for f = 1:length(TimeVector)
    DataString = [DataString sprintf('%d\t%.12f\t',f,TimeVector(f))];
    for m=1:nMarkers
        DataString = [DataString sprintf('%.12f\t%.12f\t%.12f\t',AllMarkersTRC(f,(m-1)*3+1),AllMarkersTRC(f,(m-1)*3+2),AllMarkersTRC(f,(m-1)*3+3))];
    end
    DataString = [DataString sprintf('\n')];
end
Header = strrep(Header,'DATASTART_INS',DataString);

fid = fopen(outFile,'w');
fprintf(fid,'%s',Header);
fclose(fid);
end

function header = HeaderTRC

header = ['PathFileType	4	(X/Y/Z)	FILENAME.trc' sprintf('\n') ...
    'DataRate	CameraRate	NumFrames	NumMarkers	Units	OrigDataRate	OrigDataStartFrame	OrigNumFrames' sprintf('\n') ...
    'DATARATE_INS	CAMRATE_INS	NFRAMES_INS	NMARKERS_INS	UNITS_INS	ORIGRATE_INS	1	       NFRAMES_INS' sprintf('\n') ...
    'Frame#	Time	FIRSTMARKERNAME_INS' sprintf('\n') ...
    sprintf('\t')  sprintf('\t')	'XYZ_INS' sprintf('\n') ...
    '' sprintf('\n') ...
    'DATASTART_INS' sprintf('\n')];
end

